# Modified from Jianfan: https://code.google.com/p/python-jianfan/
#
# Copyright (c) Jianfan developers.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#     1. Redistributions of source code must retain the above copyright notice,
#        this list of conditions and the following disclaimer.
#
#     2. Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#
#     3. Neither the name of Jianfan nor the names of its contributors may be used
#        to endorse or promote products derived from this software without
#        specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import codecs
import os


def _load_char_table(file_path):
    table = {}
    with codecs.open(file_path, 'r', 'utf-8') as f:
        for line in f:
            orig, replacement = line[0], line[1]
            table[orig] = replacement
    return table


dir_path = os.path.abspath(os.path.dirname(__file__))

S2T = _load_char_table(os.path.join(dir_path, 's2t.txt'))
T2S = _load_char_table(os.path.join(dir_path, 't2s.txt'))


def _translate(unistr, table):
    '''Replace characters using a table.'''
    if type(unistr) is str:
        try:
            unistr = unistr.decode('utf-8')
        # Python 3 returns AttributeError when .decode() is called on a str
        # This means it is already unicode.
        except AttributeError:
            pass
    try:
        if type(unistr) is not unicode:
            return unistr
    # Python 3 returns NameError because unicode is not a type.
    except NameError:
        pass

    chars = []
    for c in unistr:
        replacement = table.get(c)
        chars.append(replacement if replacement else c)
    return u''.join(chars)


def to_trad(text):
    '''Translate to traditional Chinese.'''
    return _translate(text, S2T)


def to_simp(text):
    '''Translate to simplified Chinese.'''
    return _translate(text, T2S)



import regex
import StringIO


regex.DEFAULT_VERSION = regex.VERSION1

# CJK characters
CHAR_CJK = (u'\u3040-\u30FF\u3105-\u312C\u3200-\u33FF\u3400-\u4DBF'
            u'\u4E00-\u9FFF\uAC00-\uD7AF\uF900-\uFAFF')

CHAR_ENG_LEFT = (u'\u0021\u0023-\u0026\\\u0029-\u005A\\\u005C-\\\u005E\u0061-\u007A'
                 u'\\\u007C\\\u007D'
                 u'\u00A2-\u00AA\u00AE-\u00BE\u00C0-\u02AF\u0370-\u052F\u1D00-\u1DBF'
                 u'\u1E00-\u1FFF\u2010-\u2016\u2019\u201D\u2020-\u2023\u2027'
                 u'\u2030-\u2034\u203A\u203C\u203D\u2043\u2044\u2046-\u2049\u205E'
                 u'\u2070-\u20CF\u2100-\u218F')

CHAR_ENG_RIGHT = (u'\u0023-\u0026\\\u0028\\\u002A\\\u002B\\\u002D\u002F-\u0039\u003C-\u003E'
                  u'\u0040-\\\u005C\\\u005E\u0061-\\\u007C'
                  u'\u00A1-\u00A9\u00AB\u00AC\u00AE-\u00AF\u00B1\u00B5-\u00B8'
                  u'\u00BC-\u02AF\u0370-\u052F\u1D00-\u1D9A\u1E00-\u1FFF'
                  u'\u2010-\u2016\u2018\u201A-\u201C\u201E-\u2023\u2027'
                  u'\u2030\u2031\u2039\u2043-\u2045\u205E\u20A0-\u20CF\u2100-\u218F')

# Patterns
PATTERN_ENG_CJK = regex.compile(u'([%s])([%s])' % (CHAR_ENG_LEFT, CHAR_CJK))
PATTERN_CJK_ENG = regex.compile(u'([%s])([%s])' % (CHAR_CJK, CHAR_ENG_RIGHT))


def add_spaces(text, exclude=None):
    if exclude:
        patt_exclude = regex.escape(exclude)
        patt_eng_cjk = regex.compile(u'([[%s]--%s])([%s])' % (CHAR_ENG_LEFT, patt_exclude, CHAR_CJK))
        patt_cjk_eng = regex.compile(u'([%s])([[%s]--%s])' % (CHAR_CJK, CHAR_ENG_RIGHT, patt_exclude))
    else:
        patt_eng_cjk = PATTERN_ENG_CJK
        patt_cjk_eng = PATTERN_CJK_ENG

    def add_space_func(index1, index2):
        def add_space(match):
            return u'%s %s' % (match.group(index1), match.group(index2))
        return add_space

    text = patt_cjk_eng.subn(add_space_func(1, 2), text)[0]
    text = patt_eng_cjk.subn(add_space_func(1, 2), text)[0]

    if not (exclude and '"' in exclude):
        # XXX"YYY"XXX -> XXX "YYY" XXX
        # where X and Y are CJK charaters
        is_left_dquote = True
        is_left_squote = True
        out = StringIO.StringIO()
        for i in xrange(len(text)):
            prev_char = text[i - 1] if i > 0 else None
            cur_char = text[i]
            next_char = text[i + 1] if i < len(text) - 1 else None
            if cur_char == u'\"':
                if is_left_dquote:
                    if _is_cjk(prev_char):
                        out.write(u' \"')
                    else:
                        out.write(u'\"')
                    is_left_dquote = False
                else:
                    if _is_cjk(next_char):
                        out.write(u'\" ')
                    else:
                        out.write(u'\"')
                    is_left_dquote = True
            elif cur_char == u'\'':
                if is_left_squote:
                    if _is_cjk(prev_char):
                        out.write(u' \'')
                    else:
                        out.write(u'\'')
                    is_left_squote = False
                else:
                    if _is_cjk(next_char):
                        out.write(u'\' ')
                    else:
                        out.write(u'\'')
                    is_left_squote = True
            else:
                out.write(cur_char)
        text = out.getvalue()
        out.close()

    return text


def _is_cjk(c):
    return ((c >= u'\u4E00' and c <= u'\u9FFF') or
            (c >= u'\u3040' and c <= u'\u30FF') or
            (c >= u'\u3105' and c <= u'\u312C') or
            (c >= u'\u3200' and c <= u'\u33FF') or
            (c >= u'\u3400' and c <= u'\u4DBF') or
            (c >= u'\uF900' and c <= u'\uFAFF'))



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import codecs
import os
import re


here = os.path.abspath(os.path.dirname(__file__))


# Read the version number from a source file.
# Why read it, and not import?
# see https://groups.google.com/d/topic/pypa-dev/0PkjVpcxTzQ/discussion
def find_version(*file_paths):
    # Open in Latin-1 so that we avoid encoding errors.
    # Use codecs.open for Python 2 compatibility
    with codecs.open(os.path.join(here, *file_paths), 'r', 'latin1') as f:
        version_file = f.read()

    # The version line must have the form
    # __version__ = 'ver'
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string')


def read_description(filename):
    with codecs.open(filename, encoding='utf-8') as f:
        return f.read()


def parse_requirements(filename):
    with open(filename) as f:
        content = f.read()
    return filter(lambda x: x and not x.startswith('#'), content.splitlines())


setup(
    name='bisheng',
    version=find_version('bisheng', '__init__.py'),
    url='https://github.com/eliangcs/bisheng',
    description='A set of utility functions for processing Chinese text.',
    long_description=read_description('README.rst'),
    author='Chang-Hung Liang',
    author_email='eliang.cs@gmail.com',
    license='BSD',
    packages=['bisheng'],
    package_data={'bisheng': ['*.txt']},
    include_package_data=True,
    install_requires=parse_requirements('requirements.txt'),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Text Processing',
        'Topic :: Text Processing :: Filters',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
